/*
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunPro, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice 
 * is preserved.
 * ====================================================
 */

/*
 * from: @(#)fdlibm.h 5.1 93/09/24
 * $NetBSD: math.h,v 1.3 1997/10/09 11:30:44 lukem Exp $
 */

#ifndef _MATH_H_
#define _MATH_H_



/*
 * ANSI/POSIX
 */
extern char __infinity[];
#define HUGE_VAL	(*(double *) __infinity)

/*
 * XOPEN/SVID
 */
#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
#define	M_E		2.7182818284590452354	/* e */
#define	M_LOG2E		1.4426950408889634074	/* log 2e */
#define	M_LOG10E	0.43429448190325182765	/* log 10e */
#define	M_LN2		0.69314718055994530942	/* log e2 */
#define	M_LN10		2.30258509299404568402	/* log e10 */
#define	M_PI		3.14159265358979323846	/* pi */
#define	M_PI_2		1.57079632679489661923	/* pi/2 */
#define	M_PI_4		0.78539816339744830962	/* pi/4 */
#define	M_1_PI		0.31830988618379067154	/* 1/pi */
#define	M_2_PI		0.63661977236758134308	/* 2/pi */
#define	M_2_SQRTPI	1.12837916709551257390	/* 2/sqrt(pi) */
#define	M_SQRT2		1.41421356237309504880	/* sqrt(2) */
#define	M_SQRT1_2	0.70710678118654752440	/* 1/sqrt(2) */

#define	MAXFLOAT	((float)3.40282346638528860e+38)
extern int32 signgam;

#if !defined(_XOPEN_SOURCE)
enum fdversion {fdlibm_ieee = -1, fdlibm_svid, fdlibm_xopen, fdlibm_posix};

#define _LIB_VERSION_TYPE enum fdversion
#define _LIB_VERSION _fdlib_version  

/* if global variable _LIB_VERSION is not desirable, one may 
 * change the following to be a constant by: 
 *	#define _LIB_VERSION_TYPE const enum version
 * In that case, after one initializes the value _LIB_VERSION (see
 * s_lib_version.c) during compile time, it cannot be modified
 * in the middle of a program
 */ 
extern  _LIB_VERSION_TYPE  _LIB_VERSION;

#define _IEEE_  fdlibm_ieee
#define _SVID_  fdlibm_svid
#define _XOPEN_ fdlibm_xopen
#define _POSIX_ fdlibm_posix

struct exception {
	int32 type;
	char *name;
	double arg1;
	double arg2;
	double retval;
};

#define	HUGE		MAXFLOAT

/* 
 * set X_TLOSS = pi*2**52, which is possibly defined in <values.h>
 * (one may replace the following line by "#include <values.h>")
 */

#define X_TLOSS		1.41484755040568800000e+16 

#define	DOMAIN		1
#define	SING		2
#define	OVERFLOW	3
#define	UNDERFLOW	4
#define	TLOSS		5
#define	PLOSS		6

#endif /* !_XOPEN_SOURCE */
#endif /* !_ANSI_SOURCE && !_POSIX_SOURCE */
#define int32_t long
#define u_int32_t unsigned long


/*
 * ANSI/POSIX
 */
 
__EXTERN double acos __PROTO((double));
__EXTERN double asin __PROTO((double));
__EXTERN double atan __PROTO((double));
__EXTERN double atan2 __PROTO((double, double));
__EXTERN double cos __PROTO((double));
__EXTERN double sin __PROTO((double));
__EXTERN double tan __PROTO((double));

__EXTERN double cosh __PROTO((double));
__EXTERN double sinh __PROTO((double));
__EXTERN double tanh __PROTO((double));

__EXTERN double exp __PROTO((double));
__EXTERN double frexp __PROTO((double, int32 *));
__EXTERN double ldexp __PROTO((double, int32));
__EXTERN double log __PROTO((double));
__EXTERN double log10 __PROTO((double));
__EXTERN double modf __PROTO((double, double *));

__EXTERN double pow __PROTO((double, double));
__EXTERN double sqrt __PROTO((double));

__EXTERN double ceil __PROTO((double));
__EXTERN double fabs __PROTO((double));
__EXTERN double floor __PROTO((double));
__EXTERN double fmod __PROTO((double, double));

#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
__EXTERN double erf __PROTO((double));
__EXTERN double erfc __PROTO((double));
__EXTERN double gamma __PROTO((double));
__EXTERN double hypot __PROTO((double, double));
__EXTERN int32 isinf __PROTO((double));
__EXTERN int32 isnan __PROTO((double));
__EXTERN int32 finite __PROTO((double));
__EXTERN double j0 __PROTO((double));
__EXTERN double j1 __PROTO((double));
__EXTERN double jn __PROTO((int32, double));
__EXTERN double lgamma __PROTO((double));
__EXTERN double y0 __PROTO((double));
__EXTERN double y1 __PROTO((double));
__EXTERN double yn __PROTO((int32, double));

#if !defined(_XOPEN_SOURCE)
__EXTERN double acosh __PROTO((double));
__EXTERN double asinh __PROTO((double));
__EXTERN double atanh __PROTO((double));
__EXTERN double cbrt __PROTO((double));
__EXTERN double logb __PROTO((double));
__EXTERN double nextafter __PROTO((double, double));
__EXTERN double remainder __PROTO((double, double));
__EXTERN double scalb __PROTO((double, double));

__EXTERN int32 matherr __PROTO((struct exception *));

/*
 * IEEE Test Vector
 */
__EXTERN double significand __PROTO((double));

/*
 * Functions callable from C, intended to support IEEE arithmetic.
 */
__EXTERN double copysign __PROTO((double, double));
__EXTERN int32 ilogb __PROTO((double));
__EXTERN double rint __PROTO((double));
__EXTERN double scalbn __PROTO((double, int32));

/*
 * BSD math library entry points
 */
struct __cabsargs { double x; double y; };
__EXTERN double cabs __PROTO((struct __cabsargs));
__EXTERN double drem __PROTO((double, double));
__EXTERN double expm1 __PROTO((double));
__EXTERN double log1p __PROTO((double));

/*
 * Reentrant version of gamma & lgamma; passes signgam back by reference
 * as the second argument; user must allocate space for signgam.
 */
#ifdef _REENTRANT
__EXTERN double gamma_r __PROTO((double, int32 *));
__EXTERN double lgamma_r __PROTO((double, int32 *));
#endif /* _REENTRANT */


/* float versions of ANSI/POSIX functions */
__EXTERN float acosf __PROTO((float));
__EXTERN float asinf __PROTO((float));
__EXTERN float atanf __PROTO((float));
__EXTERN float atan2f __PROTO((float, float));
__EXTERN float cosf __PROTO((float));
__EXTERN float sinf __PROTO((float));
__EXTERN float tanf __PROTO((float));

__EXTERN float coshf __PROTO((float));
__EXTERN float sinhf __PROTO((float));
__EXTERN float tanhf __PROTO((float));

__EXTERN float expf __PROTO((float));
__EXTERN float frexpf __PROTO((float, int32 *));
__EXTERN float ldexpf __PROTO((float, int32));
__EXTERN float logf __PROTO((float));
__EXTERN float log10f __PROTO((float));
__EXTERN float modff __PROTO((float, float *));

__EXTERN float powf __PROTO((float, float));
__EXTERN float sqrtf __PROTO((float));

__EXTERN float ceilf __PROTO((float));
__EXTERN float fabsf __PROTO((float));
__EXTERN float floorf __PROTO((float));
__EXTERN float fmodf __PROTO((float, float));

__EXTERN float erff __PROTO((float));
__EXTERN float erfcf __PROTO((float));
__EXTERN float gammaf __PROTO((float));
__EXTERN float hypotf __PROTO((float, float));
__EXTERN int32 isinff __PROTO((float));
__EXTERN int32 isnanf __PROTO((float));
__EXTERN int32 finitef __PROTO((float));
__EXTERN float j0f __PROTO((float));
__EXTERN float j1f __PROTO((float));
__EXTERN float jnf __PROTO((int32, float));
__EXTERN float lgammaf __PROTO((float));
__EXTERN float y0f __PROTO((float));
__EXTERN float y1f __PROTO((float));
__EXTERN float ynf __PROTO((int32, float));

__EXTERN float acoshf __PROTO((float));
__EXTERN float asinhf __PROTO((float));
__EXTERN float atanhf __PROTO((float));
__EXTERN float cbrtf __PROTO((float));
__EXTERN float logbf __PROTO((float));
__EXTERN float nextafterf __PROTO((float, float));
__EXTERN float remainderf __PROTO((float, float));
__EXTERN float scalbf __PROTO((float, float));

/*
 * float version of IEEE Test Vector
 */
__EXTERN float significandf __PROTO((float));

/*
 * Float versions of functions callable from C, intended to support
 * IEEE arithmetic.
 */
__EXTERN float copysignf __PROTO((float, float));
__EXTERN int32 ilogbf __PROTO((float));
__EXTERN float rintf __PROTO((float));
__EXTERN float scalbnf __PROTO((float, int32));

/*
 * float versions of BSD math library entry points
 */
struct __cabsfargs { float x; float y; };
__EXTERN float cabsf __PROTO((struct __cabsfargs));
__EXTERN float dremf __PROTO((float, float));
__EXTERN float expm1f __PROTO((float));
__EXTERN float log1pf __PROTO((float));

/*
 * Float versions of reentrant version of gamma & lgamma; passes
 * signgam back by reference as the second argument; user must
 * allocate space for signgam.
 */
#ifdef _REENTRANT
__EXTERN float gammaf_r __PROTO((float, int32 *));
__EXTERN float lgammaf_r __PROTO((float, int32 *));
#endif	/* _REENTRANT */

#endif /* !_XOPEN_SOURCE */
#endif /* !_ANSI_SOURCE && !_POSIX_SOURCE */



#endif /* _MATH_H_ */
